#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import gettext
from ikabot.config import *
from ikabot.web.sesion import *
from ikabot.helpers.gui import *
from ikabot.helpers.process import run
from ikabot.funcion.donar import donar
from ikabot.funcion.update import update
from ikabot.helpers.pedirInfo import read
from ikabot.funcion.getStatus import getStatus
from ikabot.funcion.botDonador import botDonador
from ikabot.helpers.botComm import cargarTelegram
from ikabot.helpers.signals import setSignalsHandlers
from ikabot.funcion.subirEdificio import subirEdificios
from ikabot.funcion.buscarEspacios import buscarEspacios
from ikabot.funcion.alertarAtaques import alertarAtaques
from ikabot.funcion.modoVacaciones import modoVacaciones
from ikabot.funcion.activarMilagro import activarMilagro
from ikabot.funcion.entrenarTropas import entrenarTropas
from ikabot.funcion.entrenarFlotas import entrenarFlotas
from ikabot.funcion.venderRecursos import venderRecursos
from ikabot.funcion.repartirRecurso import repartirRecurso
from ikabot.funcion.alertarPocoVino import alertarPocoVino
from ikabot.funcion.comprarRecursos import comprarRecursos
from ikabot.funcion.entrarDiariamente import entrarDiariamente
from ikabot.funcion.menuRutaComercial import menuRutaComercial
from ikabot.funcion.construirEdificio import construirEdificio
from ikabot.funcion.movimientosNavales import movimientosNavales
from ikabot.funcion.distributeResourcesEvenly import distributeResourcesEvenly

t = gettext.translation('command_line', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def menu(s):
	banner()
	menu_actions = [
					subirEdificios,
					menuRutaComercial,
					repartirRecurso,
					distributeResourcesEvenly,
					getStatus,
					donar,
					buscarEspacios,
					entrarDiariamente,
					alertarAtaques,
					botDonador,
					alertarPocoVino,
					comprarRecursos,
					venderRecursos,
					modoVacaciones,
					activarMilagro,
					entrenarTropas,
					entrenarFlotas,
					movimientosNavales,
					construirEdificio,
					update,
					cargarTelegram
					]
	print(_('(0)  Salir'))
	print(_('(1)  Lista de construcción'))
	print(_('(2)  Enviar recursos'))
	print(_('(3)  Distribuir recursos'))
	print(_('(4)  Distribuir recursos uniformemente'))
	print(_('(5)  Estado de la cuenta'))
	print(_('(6)  Donar'))
	print(_('(7)  Buscar espacios nuevos'))
	print(_('(8)  Entrar diariamente'))
	print(_('(9)  Alertar ataques'))
	print(_('(10) Donar automaticamente'))
	print(_('(11) Alertar poco vino'))
	print(_('(12) Comprar recursos'))
	print(_('(13) Vender recursos'))
	print(_('(14) Activar modo vacaciones'))
	print(_('(15) Activar milagro'))
	print(_('(16) Entrenar tropas'))
	print(_('(17) Entrenar flotas'))
	print(_('(18) Ver movimientos'))
	print(_('(19) Construir edificio'))
	print(_('(20) Actualizar Ikabot'))
	print(_('(21) Actualizar datos de Telegram'))
	entradas = len(menu_actions)
	eleccion = read(min=0, max=entradas)
	if eleccion != 0:
		try:
			menu_actions[eleccion - 1](s)
		except KeyboardInterrupt:
			pass
		menu(s)
	else:
		clear()

def inicializar():
	os.chdir(os.getenv("HOME"))
	if not os.path.isfile(ikaFile):
		open(ikaFile, 'w')
		os.chmod(ikaFile, 0o600)

def start():
	inicializar()
	s = Sesion()
	setSignalsHandlers(s)
	try:
		menu(s)
	finally:
		if os.fork() == 0:
			s.logout()

def main():
	try:
		start()
	except KeyboardInterrupt:
		clear()

if __name__ == '__main__':
	main()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import banner
from ikabot.helpers.recursos import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.process import forkear
from ikabot.helpers.varios import addPuntos
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.planearViajes import planearViajes

t = gettext.translation('distributeResourcesEvenly',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def distributeResourcesEvenly(s):

	banner() #displays the ascii art banner

	print(_('¿Qué recurso quiere distribuir?'))
	print(_('(0) Salir'))
	for i in range(4):
		print('({:d}) {}'.format(i+1, tipoDeBien[i]))
	resource = read(min=0, max=5)
	if resource == 0:
		return
	resource -= 1

	resourceTotal = 0 #total number of selected resource
	(cityIDs, cities) = getIdsDeCiudades(s) #gets city ids
	originCities = {} #dictionary for origin cities
	destinationCities = {} #dictionary for destination cities
	allCities = {} #dictionary for all cities
	for cityID in cityIDs: #for each of the users cities
		
		
		html = s.get(urlCiudad + cityID) #load html from the get request for that particular city
		city = getCiudad(html) #convert the html to a city object
		
		resourceTotal += city['recursos'][resource] #the cities resources are added to the total
		allCities[cityID] = city #adds the city to all cities
	resourceAverage = resourceTotal // len(allCities) #calculate the resource average
	resourceTotal = 0
	for cityID in cityIDs: #iterate through all the cities and exclude the ones with less capacity than the resource average from calculations
		if allCities[cityID]['capacidad'] < resourceAverage:
			allCities.pop(cityID)
		else: 
			resourceTotal += allCities[cityID]['recursos'][resource] #else add it to the total for recalculation
	resourceAverage = resourceTotal // len(allCities) #recalculate the resource average
	
	for cityID in allCities: #iterate through cities and classify them as origin or destination based on amount of resource above average
		if allCities[cityID]['recursos'][resource] > resourceAverage:
			originCities[cityID] = allCities[cityID]['recursos'][resource] - resourceAverage #sets the value to the amount of resource above average
		else: 
			destinationCities[cityID] = resourceAverage - allCities[cityID]['recursos'][resource] #sets the value to the amount of resource below average

	originCities = {k: v for k, v in sorted(originCities.items(), key=lambda item: item[1],reverse=True)} #sort origin cities in descending order
	destinationCities = {k: v for k, v in sorted(destinationCities.items(), key=lambda item: item[1])}    #sort destination cities in ascending order

	routes = []
	
	for originCityID in originCities: #iterate through all origin city ids

		for destinationCityID in destinationCities: #iterate through all destination city ids
			if originCities[originCityID] == 0 or destinationCities[destinationCityID] == 0:
				continue

			if originCities[originCityID] > destinationCities[destinationCityID]: #if there's more resources above average in the origin city than resources below average in the destination city (origin city needs to have a surplus and destination city needs to have a deficit of resources for a route to be considered)
				toSend = destinationCities[destinationCityID] #number of resources to send is the number of resources below average in destination city
			else:
				toSend = originCities[originCityID] #send the amount of resources above average of the current origin city

			# ROUTE BLOCK
			if resource == 0:
				route = (allCities[originCityID],allCities[destinationCityID],allCities[destinationCityID]['islandId'],toSend,0,0,0,0)
			elif resource == 1:
				route = (allCities[originCityID],allCities[destinationCityID],allCities[destinationCityID]['islandId'],0,toSend,0,0,0)
			elif resource == 2:
				route = (allCities[originCityID],allCities[destinationCityID],allCities[destinationCityID]['islandId'],0,0,toSend,0,0)
			elif resource == 3:
				route = (allCities[originCityID],allCities[destinationCityID],allCities[destinationCityID]['islandId'],0,0,0,toSend,0)
			elif resource == 4:
				route = (allCities[originCityID],allCities[destinationCityID],allCities[destinationCityID]['islandId'],0,0,0,0,toSend)
			routes.append(route)

			# ROUTE BLOCK
			if originCities[originCityID] > destinationCities[destinationCityID]:
				originCities[originCityID] -= destinationCities[destinationCityID] #remove the sent amount from the origin city's surplus
				destinationCities[destinationCityID] = 0 #set the amount of resources below average in destination city to 0
			else:
				destinationCities[destinationCityID] -= originCities[originCityID] #remove the sent amount from the amount of resources below average in current destination city
				originCities[originCityID] = 0 #set the amount of resources above average in origin city to 0

	banner()
	print(_('\nSe realizarán los siguientes envios:\n'))
	for route in routes:
		print('{} -> {} : {} {}'.format(route[0]['name'], route[1]['name'], route[resource+3], tipoDeBien[resource])) #displays all routes to be executed in console

	print(_('\nTodas las ciuaddes tendrán al rededor de {} de {}').format(addPuntos(resourceAverage), tipoDeBien[resource]))

	print(_('\n¿Proceder? [Y/n]'))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		return

	forkear(s) #forks the process.
	if s.padre is True:
		return

	info = _('\nDistribuyo {} de forma uniforme entre todas las ciudades\n').format(tipoDeBien[resource])
	setInfoSignal(s, info)

	try:
		planearViajes(s, routes) #plan trips for all the routes
	except:
		msg = _('Error en:\n{}\nCausa:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg) #sends message to telegram bot
	finally:
		s.logout()
